
package Gtk2::BindScalar;

# This is a generic base class for creating tie'd variables linked to
# Gtk2 widgets.
#
#
# Basic methodology: we provide the basic requirements of the TIE
# here, and rely on names of the get/set functions passed to the ctor
# to implement the specifics of the get_value() and set_value()
# functions.
#
# For the most part, we rely on a standard get_<thingy>/set_<thingy>
# convention that is followed for the most part with the Gtk2 widgets.
#
# For widgets that don't support that, the basic implementation works
# in terms of an accessor/modifier pair of function names that may be
# passed in the constructor

use Carp;

my %connInfo = ( 
				'Gtk2::CheckButton' => [ ['toggled'], ['active'] ],
				'Gtk2::CheckMenuItem' => [ ['toggled'], ['active'] ],
				'Gtk2::Entry' => [ ['focus-out-event'], ['text'] ],
				'Gtk2::Notebook' => [ ['switch-page','change-current-page'],
									  ['current_page'] ],
				'Gtk2::OptionMenu' => [ ['changed'], ['history'] ],
				'Gtk2::Range' => [ ['value-changed'], ['value'] ],
				'Gtk2::SpinButton' => [ ['value-changed'], ['value'] ],
				'Gtk2::TextView' => [ ['focus-out-event'], ['buffer'] ],
				'Gtk2::ToggleButton' => [ [ 'toggled' ], ['active'] ],
			   );

sub new {
  my ($class, $ref, $widg, @rest) = @_;
  my $info = $connInfo{ref($widg)};
  if(!defined $info || (@_ < 3)) {
	use Data::Dumper;
	print Dumper(\%connInfo), "\n";
	croak "Usage: $_[0]\->new (\\var, widget)\n"
		. " expecting a reference to the storage variable"
		. " and the widget to which to bind.\n"
		. "Possibly ".ref($widg)." was not found, did you register with\n"
		." the %Gtk2::BindScalar::connInfo hash?";
  }
  if(scalar(@{$info->[1]}) == 2){
	$getFunc = $info->[1]->[0];
	$setFunc = $info->[1]->[1];
  }else{
	  # assume we are using the property ctor shortcut
	my $prop = $info->[1]->[0];
	$setFunc = "set_$prop";
	$getFunc = "get_$prop";
  }
  my $obj =  bless { 'widg' => $widg,
					 'var' => $ref,
					 'get' => $getFunc,
					 'set' => $setFunc }, $class;

  foreach my $s (@{$info->[0]}){
	$widg->signal_connect($s, sub { $obj->update_var(); });
  }

  $obj->update_widget();
  return $obj
}

sub register_connection_info {
  my $name = shift;
  my $info = shift;
  if(defined $connInfo{$name}) {
	carp "Gtk2::BindScalar::register_connection_info: [$name] already registered\n";
  }
  $connInfo{$name} = $info;
  return 1;
}

sub bound_widget {
  return $_[0]->{'widg'};
}

sub bind_var {
  my $self = shift;
  my $var = shift;

  $self->{'var'} = $var;
  $self->update_widget();
}


sub update_var {
  my $self = shift;
  my $func = $self->{'get'};
  if(ref($func) eq 'CODE'){
	${$self->{'var'}} = &{$func}($self->{'widg'});
  }else{
	${$self->{'var'}} = $self->{'widg'}->$func();
  }
  return 0;
}

sub update_widget {
  my $self = shift;

  my $func = $self->{'set'};
  if(defined ${$self->{'var'}}){
	if(ref($func) eq 'CODE'){
	  &{$func}($self->{'widg'}, ${$self->{'var'}});
	}else{
	  $self->{'widg'}->$func(${$self->{'var'}});
	}
  }
}


1;

__END__
# documentation is a good thing.

=head1 NAME

Gtk2::BindScalar - A simple bind class to link data var to widget contents

=head1 SYNOPSIS

  use Gtk2 '-init';
  use Gtk2::BindScalar;

  my $widg = Gtk2::Entry->new();

  $widg->set_text("hello");

  my $var = "hola";

  my $conn = Gtk2::BindScalar->create(\$var, $widg);

  print "Widg: ", $widg->get_text(), " Var: $var\n";
  $var = "bienvenue";  $conn->update_widget();
  print "Widg: ", $widg->get_text(), " Var: $var\n";
  $widg->set_text("welkom");
  print "Widg: ", $widg->get_text(), " Var: $var\n";

  # break connection
  $conn->destroy();   # or (tied $var)->destroy();
  $widg->set_text("hoi");
  print "Widg: ", $widg->get_text(), " Var: $var\n";


=head1 ABSTRACT

BindScalar is a simple interface for creating a connection between a
variable and a widget contents.

=head1 DESCRIPTION

Gtk2 has a powerful, but complex widget structure.  Gtk2::BindScalar
provides a simplified interface for the common task of populating and
retrieving the content of a widget.  This allows a cleaner separation
of widget from program logic, once the variable is tied to the widget,
the reference to the widget itself is no longer necessary for simple
operation.

The variable's value is updated as soon as the widget is manipulated
by the user, but is not so closely linked as a tie'd variable is (see
Gtk2::TieScalar) so if the widget is destroyed, the value remains.

=head1 OBJECT HIERARCHY

 Gtk2::BindScalar

=head1 METHODS

=over

=item $conn = Gtk2::BindScalar->new(var, widget)

Binds the C<var> to the C<widget>.  As the widget is manipulated, the
variable is updated.


=item $conn->bind_var()

Rebinds the connection between the widget to a new variable.  Changes
to either the widget or the variable no longer affect the original.
Updates the widget state at the same time to reflect new contents.


=item Gtk2::BindScalar::update_var()

Update the value to reflect current widget contents.  This is the
function a derived class would override to change access methods.


=item Gtk2::BindScalar::update_widget()

Update the widget contents to reflect the current variable's value.
This is the function a derived class would override to change access
methods.


=head1 SEE ALSO

Perl(1), Gtk2(3pm).

=head1 AUTHORS

 Carl Nygard <cjnygard at fast dot net>
   inspiration credit to Bjarne Steinsbo <bosteins at broadpark dot no>

=head1 COPYRIGHT AND LICENSE

Copyright 2004 by Spatial Software Solutions.

This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Library General Public License as published by the Free
Software Foundation; either version 2.1 of the License, or (at your option) any
later version.

This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Library General Public License for more
details.

You should have received a copy of the GNU Library General Public License along
with this library; if not, write to the Free Software Foundation, Inc., 59
Temple Place - Suite 330, Boston, MA  02111-1307  USA.

=cut
