
package Gtk2::TieArrayRef;

# This is a generic base class for creating tie'd variables linked to
# Gtk2 widgets.
#
#
# Basic methodology: we provide the basic requirements of the TIE
# here, and rely on names of the get/set functions passed to the ctor
# to implement the specifics of the get_value() and set_value()
# functions.
#
# For the most part, we rely on a standard get_<thingy>/set_<thingy>
# convention that is followed for the most part with the Gtk2 widgets.
#
# For widgets that don't support that, the basic implementation works
# in terms of an accessor/modifier pair of function stubs that may be
# passed in the constructor

our @ISA = 'Gtk2::TieScalar';
use Gtk2::TieScalar;
use Carp;

my %defaultFunc = ( 
				   'Gtk2::SimpleList' => ['get_selected_indices', 'select' ],
				  );

sub create {
  my ($class, $ref, $widg, $getFuncName, $setFuncName, @rest) = @_;
  if(!defined $setFuncName){
	my $prop = $getFuncName;
	if(!defined $prop){
	  $prop = $defaultFunc{ref($widg)};
	}
	if(!defined $prop) {
	  croak "Usage: $_[0]\->new (\\var, widget, getFunc, setFunc)\n"
		  . " expecting a reference to the storage variable.\n"
		  . " along with the widget and the get/set function names.\n";
	}
	if(ref($prop) eq 'ARRAY') {
	  $getFuncName = $prop->[0];
	  $setFuncName = $prop->[1];
	}else{
	  # assume we are using the property ctor shortcut
	  $setFuncName = "set_$prop";
	  $getFuncName = "get_$prop";
	}
  }
  return bless Gtk2::TieScalar->create($ref, $widg,$getFuncName, $setFuncName),
	  $class;
}


sub get_value {
  my $self = shift;
  my $func = $self->{'get'};
  if(ref($func) eq 'CODE'){
	return &{$func}($self->{'widg'});
  }else{
	return [ $self->{'widg'}->$func() ];
  }
}

sub set_value {
  my $self = shift;
  my $value = shift;
  my $func = $self->{'set'};
  if(ref($func) eq 'CODE'){
	&{$func}($self->{'widg'}, $value);
  }else{
	$self->{'widg'}->$func(@$value);
  }
}

1;

__END__
# documentation is a good thing.

=head1 NAME

Gtk2::TieArrayRef - A simple tie class to link data var to widget
contents, customized for dealing with array references

=head1 SYNOPSIS

  use Gtk2 '-init';
  use Gtk2::TieArrayRef;

  # populated list widget
  my $widg = Gtk2::SimpleList->new( ... );

  $data->select(1);

  my $sel = [ 3,4,5 ];

  my $conn = Gtk2::TieArrayRef->create(\$var, $widg);

  print "Widg: ", join(",",$widg->get_selected_indices()), 
     " Var: ", join(",",@$var), "\n";
  $var = [ 2,3,5,7 ];
  print "Widg: ", join(",",$widg->get_selected_indices()), 
     " Var: ", join(",",@$var), "\n";
  $widg->get_selection()->clear_selection();
  print "Widg: ", join(",",$widg->get_selected_indices()), 
     " Var: ", join(",",@$var), "\n";

  # break connection
  $conn->destroy();   # or (tied $var)->destroy();
  $widg->select(1,2,4,8);
  print "Widg: ", join(",",$widg->get_selected_indices()), 
     " Var: ", join(",",@$var), "\n";
  $var = [ 0,1,2,3 ];
  print "Widg: ", join(",",$widg->get_selected_indices()), 
     " Var: ", join(",",@$var), "\n";


=head1 ABSTRACT

TieArrayRef is a simple interface for creating a tie between a
variable and a widget attribute.  Typically used to provide convenient
access to a widgets contents without requiring a reference to the
widget itself.

=head1 DESCRIPTION

Gtk2 has a powerful, but complex widget structure.  Gtk2::TieArrayRef
provides a simplified interface for the common task of populating and
retrieving the content of a widget.  This allows a cleaner separation
of widget from program logic, once the variable is tied to the widget,
the reference to the widget itself is no longer necessary for simple
operation.

This widget is derived from Gtk2::TieScalar, and re-implements the
get_value and set_value for specific functionality when dealing with a
reference to an array instead of a normal scalar.

=head1 OBJECT HIERARCHY

 Gtk2::TieScalar
    +--- Gtk2::TieArrayRef

=head1 METHODS

=over

=item $conn = Gtk2::TieArrayRef->create(var, widget, [getFunc, setFunc])

Ties the C<var> to the C<widget>.  If no function names are specified,
a default is looked up according to the type of C<widget>.  If
C<getFunc> is specified but no C<SetFunc>, then the get/set functions
are constructed with the patter "set_<setFunc>" and "get_<getFunc>".
This allows convenient shorthand for the most common cases, while
still providing functionality for cases where the functions are not
symmetrically named.

C<getFunc> and/or C<setFunc> can be anonymous closure functions.  In
this case, the widget is passed as the first parameter, and the value
(for the set function) is passed as the second value.  The value
passed is an array ref, and not converted to an array.  Likewise, the
getFunc should return an array ref, it will not be converted by
get_value() function.


=item $conn->destroy()

Severs the tie connection between the widget and the variable.
Changes to either the widget or the variable no longer affect the
other.


=item Gtk2::TieScalar::get_value()

Return the value using the widget and the getFunc.  This is the
function called by the FETCH interface of a tied scalar.  This is the
function a derived class would override to change access methods.

This function returns a converted array ref and assumes the get
function returns an array.

If the getFunc is a code reference, then the function should return an
array ref directly, as the get_value() function will not do any
conversion.

=item Gtk2::TieScalar::set_value($val)

Set the value using the widget and the setFunc.  This function
converts the array ref to a real array and passes it to the set
function.  Thisis the function called by the STORE interface of a tied
scalar.  This is the function a derived class would override to change
access methods.

This function passes the array ref as an array if the setFunc is a
function name.

If the setFunc is a code reference (anonymous closure, etc.), then the
function will be passed the array ref directly, since set_value() will
not do any conversion to a normal array in the function call arg list.

=head1 SEE ALSO

Perl(1), Gtk2(3pm), Gtk2::TieScalar(3pm).

=head1 AUTHORS

 Carl Nygard <cjnygard at fast dot net>

=head1 COPYRIGHT AND LICENSE

Copyright 2004 by Spatial Software Solutions.

This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Library General Public License as published by the Free
Software Foundation; either version 2.1 of the License, or (at your option) any
later version.

This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Library General Public License for more
details.

You should have received a copy of the GNU Library General Public License along
with this library; if not, write to the Free Software Foundation, Inc., 59
Temple Place - Suite 330, Boston, MA  02111-1307  USA.

=cut
