
package Gtk2::TieScalar;

# This is a generic base class for creating tie'd variables linked to
# Gtk2 widgets.
#
#
# Basic methodology: we provide the basic requirements of the TIE
# here, and rely on names of the get/set functions passed to the ctor
# to implement the specifics of the get_value() and set_value()
# functions.
#
# For the most part, we rely on a standard get_<thingy>/set_<thingy>
# convention that is followed for the most part with the Gtk2 widgets.
#
# For widgets that don't support that, the basic implementation works
# in terms of an accessor/modifier pair of function names that may be
# passed in the constructor

use Carp;

my %defaultFunc = ( 
				   'Gtk2::Button' => 'label',
				   'Gtk2::CheckButton' => 'active',
				   'Gtk2::CheckMenuItem' => 'active',
				   'Gtk2::ColorSelection' => 'current_color',
				   'Gtk2::Entry' => 'text',
				   'Gtk2::FileSelection' => 'filename',
				   'Gtk2::FontSelection' => 'font_name',
				   'Gtk2::FontSelectionDialog' => 'font_name',
				   'Gtk2::Frame' => 'label',
				   'Gtk2::Image' => 'image',
				   'Gtk2::Label' => 'text',
				   'Gtk2::Menu' => 'active',
				   'Gtk2::MenuItem' => 'submenu',
				   'Gtk2::Paned' => 'position',
				   'Gtk2::Notebook' => 'current_page',
				   'Gtk2::OptionMenu' => 'menu',
				   'Gtk2::ProgressBar' => 'fraction',
				   'Gtk2::Range' => 'value',
				   'Gtk2::SimpleList' => 
				   [ 
					sub { my @sel = $_[0]->get_selected_indices();
						  return $sel[0];
						},
					'select'
				   ],
				   'Gtk2::SpinButton' => 'value',
				   'Gtk2::TextView' => 'buffer',
				   'Gtk2::ToggleButton' => 'active',
				   'Gtk2::Widget' => 'name',
				   'Gtk2::Window' => 'title',
				  );

sub create {
  my ($class, $ref, $widg, $getFuncName, $setFuncName, @rest) = @_;
  if(!defined $setFuncName){
	my $prop = $getFuncName;
	if(!defined $prop){
	  $prop = $defaultFunc{ref($widg)};
	}
	if(!defined $prop) {
	  croak "Usage: $_[0]\->new (\\var, widget, getFunc, setFunc)\n"
		  . " expecting a reference to the storage variable.\n"
		  . " along with the widget and the get/set function names.\n";
	}
	if(ref($prop) eq 'ARRAY') {
	  $getFuncName = $prop->[0];
	  $setFuncName = $prop->[1];
	}else{
	  # assume we are using the property ctor shortcut
	  $setFuncName = "set_$prop";
	  $getFuncName = "get_$prop";
	}
  }
  my $obj =  bless { 'widg' => $widg,
					 'var' => $ref,
					 'get' => $getFuncName,
					 'set' => $setFuncName }, $class;

  my $val = $$ref;
  tie $$ref, 'Gtk2::TieScalar', $obj;
  $obj->set_value($val);
  return $obj
}

sub destroy {
  my $self = shift;
  untie ${$self->{'var'}} if(tied ${$self->{'var'}});
}


sub get_value {
  my $self = shift;
  my $func = $self->{'get'};
  if(ref($func) eq 'CODE'){
	return &{$func}($self->{'widg'});
  }else{
	return $self->{'widg'}->$func();
  }
}

sub set_value {
  my $self = shift;
  my $value = shift;
  my $func = $self->{'set'};
  if(ref($func) eq 'CODE'){
	&{$func}($self->{'widg'}, $value);
  }else{
	$self->{'widg'}->$func($value);
  }
}

sub TIESCALAR {
  my ($class, $self, @rest) = @_;
  return bless $self, $class;
}

sub FETCH {
  $_[0]->get_value();
}

sub STORE {
  $_[0]->set_value($_[1]);
}

1;

__END__
# documentation is a good thing.

=head1 NAME

Gtk2::TieScalar - A simple tie class to link data var to widget contents

=head1 SYNOPSIS

  use Gtk2 '-init';
  use Gtk2::TieScalar;

  my $widg = Gtk2::Entry->new();

  $widg->set_text("hello");

  my $var = "hola";

  my $conn = Gtk2::TieScalar->create(\$var, $widg);

  print "Widg: ", $widg->get_text(), " Var: $var\n";
  $var = "bienvenue";
  print "Widg: ", $widg->get_text(), " Var: $var\n";
  $widg->set_text("welkom");
  print "Widg: ", $widg->get_text(), " Var: $var\n";

  # break connection
  $conn->destroy();   # or (tied $var)->destroy();
  $widg->set_text("hoi");
  print "Widg: ", $widg->get_text(), " Var: $var\n";
  $var = "w31c0m3";


=head1 ABSTRACT

TieScalar is a simple interface for creating a tie between a variable
and a widget attribute.  Typically used to provide convenient access
to a widgets contents without requiring a reference to the widget
itself.

=head1 DESCRIPTION

Gtk2 has a powerful, but complex widget structure.  Gtk2::TieScalar
provides a simplified interface for the common task of populating and
retrieving the content of a widget.  This allows a cleaner separation
of widget from program logic, once the variable is tied to the widget,
the reference to the widget itself is no longer necessary for simple
operation.

=head1 OBJECT HIERARCHY

 Gtk2::TieScalar

=head1 METHODS

=over

=item $conn = Gtk2::TieScalar->create(var, widget, [getFunc, setFunc])

Ties the C<var> to the C<widget>.  If no function names are specified,
a default is looked up according to the type of C<widget>.  If
C<getFunc> is specified but no C<setFunc>, then the get/set functions
are constructed with the patter "set_<setFunc>" and "get_<getFunc>".
This allows convenient shorthand for the most common cases, while
still providing functionality for cases where the functions are not
symmetrically named.

C<getFunc> and/or C<setFunc> can be anonymous closure functions.  In
this case, the widget is passed as the first parameter, and the value
(for the set function) is passed as the second value.


=item $conn->destroy()

Severs the tie connection between the widget and the variable.
Changes to either the widget or the variable no longer affect the
other.


=item Gtk2::TieScalar::get_value()

Return the value using the widget and the getFuncName.  This is the
function called by the FETCH interface of a tied scalar.  This is the
function a derived class would override to change access methods.


=item Gtk2::TieScalar::set_value($val)

Set the value using the widget and the setFuncName.  This is the
function called by the STORE interface of a tied scalar.  This is the
function a derived class would override to change access methods.


=head1 SEE ALSO

Perl(1), Gtk2(3pm).

=head1 AUTHORS

 Carl Nygard <cjnygard at fast dot net>
   inspiration credit to Bjarne Steinsbo <bosteins at broadpark dot no>

=head1 COPYRIGHT AND LICENSE

Copyright 2004 by Spatial Software Solutions.

This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Library General Public License as published by the Free
Software Foundation; either version 2.1 of the License, or (at your option) any
later version.

This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Library General Public License for more
details.

You should have received a copy of the GNU Library General Public License along
with this library; if not, write to the Free Software Foundation, Inc., 59
Temple Place - Suite 330, Boston, MA  02111-1307  USA.

=cut
